@@ -1,9 +1,29 @@
+"""
+figenv is a metaclass that can transform config objects to be able to pull
+updates from environment variables.
+
+.. code-block:: python
+
+    class Config(metaclass=MetaConfig):
+        USERNAME = 'fake'
+        DEBUG = False
+"""
 import json
 import os
 
 _MISSING = object()
 
 
+def _check_special_names(name):
+    return name in ('name', 'keys') or name.startswith('_')
+
+
+def strict(f):
+    """Decorator to disable automatic override of dynamic configuration via environmental variable"""
+    f._strict = True
+    return f
+
+
 class MetaConfig(type):
     def __init__(cls, name, bases, dict):
         super().__init__(name, bases, dict)
@@ -16,12 +36,12 @@ class MetaConfig(type):
         cls._dict.update(dict)
 
     def __getattribute__(cls, name):
-        '''
+        """
         Return value of ``name`` and ``_dict``
 
         Fall back to getattr for everything else
-        '''
-        if name in ('name', 'keys') or name.startswith('_'):
+        """
+        if _check_special_names(name):
             return super().__getattribute__(name)
         raise AttributeError('Fallback to environment')
 
@@ -55,23 +75,36 @@ class MetaConfig(type):
     def _to_dict(cls, value):
         return json.loads(value)
 
+    def __setattr__(cls, name, value):
+        """
+        Do not allow setting attributes.  Variables should be set at creation
+        time and overwritten with environment variables in the shell
+        """
+        if not _check_special_names(name):
+            raise NotImplementedError(f'{cls} is a Immutable Type. Override using Environment Variables')
+        return super().__setattr__(name, value)
+
     def __getattr__(cls, name):
-        '''
+        """
         Check if attribute is available on the class. If it is, then check
         the environment variables for that. If it is not in the environment
         variables, then return the default set on the class. Otherwise raise
         an AttributeError.
-        '''
+        """
         prefix = cls._dict.get('ENV_PREFIX', '')
         load_all = cls._dict.get('ENV_LOAD_ALL', False)
 
         if (not load_all and name not in cls._dict) or (name not in cls._dict and prefix + name not in os.environ):
             raise AttributeError(f"type object {cls.name} has no attribute '{name}'")
 
-        if prefix + name in os.environ:
+        value = cls._dict.get(name, None)
+
+        override_via_environment = True
+        if callable(value) and getattr(value, "_strict", False):
+            override_via_environment = False
+
+        if override_via_environment and prefix + name in os.environ:
             value = os.environ[prefix + name]
-        else:
-            value = cls._dict[name]
 
         if callable(value):
             value = value(cls)
